###############################################################################
#
#  Copyright (C) 2007-TODAY OpenERP SA. All Rights Reserved.
#
#  $Id$
#
#  Developed by OpenERP (http://openerp.com) and Axelor (http://axelor.com).
#
#  The OpenERP web client is distributed under the "OpenERP Public License".
#  It's based on Mozilla Public License Version (MPL) 1.1 with following 
#  restrictions:
#
#  -   All names, links and logos of OpenERP must be kept as in original
#      distribution without any changes in all software screens, especially
#      in start-up page and the software header, even if the application
#      source code has been changed or updated or code has been added.
#
#  You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################
import copy
import base64
import json

from openerp.controllers import SecuredController
from openerp.utils import rpc, cache, common, TinyDict

from openobject.tools import expose


#change 'en' to false for context
def adapt_context(val):
    if val == 'en_US':
        return False
    else:
        return val

class Translator(SecuredController):

    _cp_path = "/openerp/translator"

    @expose(template="/openerp/controllers/templates/translator.mako")
    def index(self, translate='fields', **kw):
        params, data = TinyDict.split(kw)
        
        ctx = dict((params.context or {}), **rpc.session.context)
        params['context'] = ustr(ctx)

        proxy = rpc.RPCProxy('res.lang')

        lang_ids = proxy.search([('translatable', '=', '1')])
        langs = proxy.read(lang_ids, ['code', 'name'])

        proxy = rpc.RPCProxy(params.model)

        data = []
        view = []

        fields_type = {}
        view_view = cache.fields_view_get(params.model, False, 'form', ctx, True)

        view_fields = view_view['fields']
        view_relates = view_view.get('toolbar')

        names = view_fields.keys()
        names.sort(lambda x,y: cmp(view_fields[x].get('string', ''), view_fields[y].get('string', '')))

        if translate == 'fields' and params.id:
            # US-3071 : single field translation only for product.product
            if params.get('_terp_clicked_field', False) and params.get('_terp_model', False) \
                    and params['_terp_model'] == 'product.product':
                clicked_field = params['_terp_clicked_field']
                attrs = view_fields[clicked_field]
                if attrs.get('translate'):
                    fields_type[clicked_field] = attrs.get('type')
                    value = {}
                    for lang in langs:
                        context = copy.copy(ctx)
                        context['lang'] = adapt_context(lang['code'])

                        val = proxy.read([params.id], [clicked_field], context)
                        val = val[0] if isinstance(val,list) and len(val) > 0 else None

                        value[lang['code']] = val[clicked_field] if isinstance(val,dict) \
                            and clicked_field in val else None

                    data += [(clicked_field, value, None, attrs.get('string'))]
            else:
                for name in names:
                    attrs = view_fields[name]
                    if attrs.get('translate'):
                        value = {}
                        for lang in langs:
                            context = copy.copy(ctx)
                            context['lang'] = adapt_context(lang['code'])

                            val = proxy.read([params.id], [name], context)
                            val = val[0] if isinstance(val, list) and len(val) > 0 else None

                            value[lang['code']] = val[name] if isinstance(val, dict) \
                                                               and name in val else None

                        data += [(name, value, None, attrs.get('string'))]

        if translate == 'labels':
            for name in names:
                attrs = view_fields[name]
                if attrs.get('string'):
                    value = {}
                    for lang in langs:
                        code=lang['code']
                        val = proxy.read_string(False, [code], [name])

                        if name in val[code]:
                            value[code] = val[code][name] or None

                    if value: data += [(name, value, None, None)]

        if translate == 'relates' and view_relates:
            for bar, tools in view_relates.items():
                for tool in tools:

                    value = {}
                    for lang in langs:
                        code = lang['code']
                        val = rpc.session.execute('object', 'execute', tool['type'], 'read', [tool['id']], ['name'], {'lang': code})

                        value[code] = val[0]['name'] or None

                    data += [(tool['id'], value, tool['type'], None)]

        if translate == 'view':
            for lang in langs:
                code=lang['code']
                view_item_ids = rpc.session.execute('object', 'execute', 'ir.translation', 'search', [('name', '=', params.model), ('type', '=', 'view'), ('lang', '=', code)])
                view_items = rpc.session.execute('object', 'execute', 'ir.translation', 'read', view_item_ids, ['src', 'value'])

                values = []
                for val in view_items:
                    values += [val]

                if values:
                    view += [(code, values)]

        return dict(translate=translate, langs=langs, data=data, view=view, model=params.model, id=params.id, fields_type=fields_type, ctx=params.context)

    @expose(template="/openerp/controllers/templates/translator.mako")
    def save(self, translate='fields', **kw):
        params, data = TinyDict.split(kw)
        
        ctx = dict((params.context or {}), **rpc.session.context)
        params['context'] = ustr(ctx)
        fields_values = []

        if translate == 'fields':
            if not params.id:
                raise common.message(_("You need to save the resource before adding translations."))

            for lang, value in data.items():

                context = copy.copy(ctx)
                context['lang'] = adapt_context(lang)

                for name, val in value.items():
                    if isinstance(val, basestring):
                        val = [val]

                    for v in val:
                        if not v:
                            # look for EN value first, then any
                            if data['en_MF'] and data['en_MF'][name]:
                                v = data['en_MF'][name]
                                data[context['lang']][name] = data['en_MF'][name]
                            else:
                                for lang in data:
                                    if data[lang] and data[lang][name]:
                                        v = data[lang][name]
                                        data[context['lang']][name] = data[lang][name]
                                        break
                        rpc.session.execute('object', 'execute', params.model, 'write', [params.id], {name : v}, context)

            for lang_field in data[ctx['lang']]:
                fields_values.append({
                    'id': lang_field,
                    'value': data[ctx['lang']][lang_field],
                })

        if translate == 'labels':
            for lang, value in data.items():
                for name, val in value.items():
                    rpc.session.execute('object', 'execute', params.model, 'write_string', False, [lang], {name: val})

        if translate == 'relates':
            for lang, value in data.items():
                for name, val in value.items():
                    rpc.session.execute('object', 'execute', params.models[name], 'write', [int(name)], {'name': val}, {'lang': lang})

        if translate == 'view':
            for lang, value in data.items():
                for id, val in value.items():
                    rpc.session.execute('object', 'execute', 'ir.translation', 'write', [int(id)], {'value': val})

        # Encoding fields_value to prevent escaping characters and shorten sent data
        return dict(close_popup=True, fields_values=base64.b64encode(json.dumps(fields_values)))

# vim: ts=4 sts=4 sw=4 si et
